package ru.amse.timkina.archiver.filetree;
import java.util.ArrayList;
import java.util.List;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class Directory extends File implements IDirectory {
	
	private final List<IFile> myFiles = new ArrayList<IFile>();
	private final List<IDirectory> myDirectories = new ArrayList<IDirectory>();
	private IDirectory myParent;
	private final Map<String, IDirectory> myKids = new HashMap<String, IDirectory> ();
	public boolean hasUp = false;
		
	public Directory() {
		super(0);
		myParent = this;
	}
	
	public IDirectory getParent() {
		return myParent;
	}
	
	public void addUp() {
		if (!hasUp){
			IDirectory up = new Directory();
			up.setName("..");
		    up.setParent(up);
		    myDirectories.add(0, up);
		    hasUp = true;
		}	
	}
	
	public boolean hasUp() {
		return hasUp;
	}
	
	public void setParent(IDirectory directory) {
		myParent = directory;
	}
	
	public void setSize(long size) {
		mySize = size;
	}
	
	public long getSize() {
		long sum = 0;
		if((myFiles.size() == 0) && (myDirectories.size() == 0)) {
			return 0;
		} else {			
			for(int i = 0; i < myFiles.size(); i++) {
				sum += myFiles.get(i).getSize();
			}
			for(int j = 0; j < myDirectories.size(); j++) {
				sum += myDirectories.get(j).getSize();
			}
		}
		return sum;
	}
	
	public int getContentSize() {
		return myFiles.size() + myKids.size();
	}
	
	public void add(IFile node) {
		if (node.isDirectory()) {
			myDirectories.add((Directory) node);	
			((Directory) node).setParent(this);			
			if (!(node.getName() == "..")) {
				myKids.put(node.getShortName(), (Directory) node);			
			}
		} else {
			myFiles.add(node);
			node.setParent(this);
		}		
	}

	public void delete(IFile node) {
		if (node.isDirectory()) {
			myDirectories.remove(node);		
			myKids.remove(node.getShortName());
		} else {
			myFiles.remove(node);
		}	
		
	}
	
	public boolean isDirectory() {
		return true;
	}
	
	public List<IFile> getFiles() {
		return Collections.unmodifiableList(myFiles);
	}
	
	public List<IDirectory> getDirectories() {
		return Collections.unmodifiableList(myDirectories);
	}

	public IDirectory getDirectoryByName(String name) {
		return myKids.get(name);
	}
	
	public boolean containsDirectory(String name) {
		return myKids.containsKey(name);
	}
	
	

}
