package ru.amse.timkina.archiver.ui.menu;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import java.io.IOException;
import javax.swing.AbstractAction;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import ru.amse.timkina.archiver.filetree.IDirectory;
import ru.amse.timkina.archiver.filetree.IFile;
import ru.amse.timkina.archiver.ui.ArchiverFrame;
import ru.amse.timkina.archiver.ui.table.MainTable;
import ru.amse.timkina.archiver.ui.table.MainTableModel;
import ru.amse.timkina.archiver.zipprocessor.Unzipper;

public class UnzipAction extends AbstractAction {
	
	private File myCurrentDir;
	private final MainTable myTable;
	private final Unzipper myUnzipper = new Unzipper();
	private final ArchiverFrame myFrame;

	public UnzipAction(MainTable table, ArchiverFrame frame) {
		putValue(NAME, "Unzip");
		putValue(SHORT_DESCRIPTION, "Unzip selected files");
		myFrame = frame;
		URL unzipURL = myFrame.getClass().getResource("Unzip.jpg");	
		ImageIcon unzip = new ImageIcon(unzipURL);
		putValue(SMALL_ICON, unzip);
		myTable = table;		
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		final String name = ((MainTableModel) (myTable.getModel())).getArchive().getName();
		if (name == null) {
			JOptionPane.showMessageDialog(myFrame, "Open archive, please");
		} else {
			if (myTable.getSelectedRows().length == 0) {
				JOptionPane.showMessageDialog(myFrame, 
						"Select files in the table, please");
			} else {
				JFileChooser fc = new JFileChooser();
				fc.setCurrentDirectory(myCurrentDir);
				fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				fc.setDialogTitle("Folder where to unzip");
				int res = fc.showOpenDialog(fc);		
				if (res == JFileChooser.APPROVE_OPTION) {
					myCurrentDir = fc.getSelectedFile();
			        int[] rowsSelected = myTable.getSelectedRows();		       
			        MainTableModel model = (MainTableModel) myTable.getModel();
			        IDirectory cur = model.getDirectory();
			        final List<IFile> filesSelected = new ArrayList<IFile>();
			        for (int i = 0; i < rowsSelected.length; i++) {
			        	int num;
				        if ((num = rowsSelected[i]) != 0) {
				    	    IFile file;
				    	    List<IDirectory> dirs = cur.getDirectories();
				    	    List<IFile> files = cur.getFiles();				
				    	    if (num <= dirs.size() - 1) {
				            	file = dirs.get(num);
			        	    } else {
			    	    	    file = files.get(num - dirs.size());
				            }
					        filesSelected.add(file);
				        }			
			        }
			    	final JDialog dialog = new JDialog(myFrame, "Wait, please", false);
			        dialog.setLocationRelativeTo(myFrame);
			        dialog.setSize(200, 150);
			        dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			        JLabel label = new JLabel("Unzipping", JLabel.CENTER);
			        label.setOpaque(true);
			        dialog.add(label, BorderLayout.CENTER);	    	
			        dialog.setVisible(true);
			        dialog.repaint();
			        SwingUtilities.invokeLater(new Runnable() {		    	
			        	@Override
			        	public void run() {
			        		try {
			        			myUnzipper.unzip(name, filesSelected, myCurrentDir.getAbsolutePath());
			        			dialog.dispose();
			        		} catch (IOException ex) {
			        			JOptionPane.showMessageDialog(myFrame, ex.getMessage());
			        		}
			        	}
			        });	
				}
			}			
		}
	} 
}
