package ru.amse.timkina.archiver.ui.table;

import javax.swing.table.AbstractTableModel;
import javax.swing.event.*;
import java.text.SimpleDateFormat;

import ru.amse.timkina.archiver.filetree.IFile;
import ru.amse.timkina.archiver.filetree.IDirectory;
import ru.amse.timkina.archiver.ui.table.viewobjects.NameView;
import ru.amse.timkina.archiver.ui.table.viewobjects.SizeConverter;
import ru.amse.timkina.archiver.ui.table.viewobjects.SizeView;
import ru.amse.timkina.archiver.ui.table.viewobjects.TimeView;

public class MainTableModel extends AbstractTableModel {

	private final String[] myColumnNames = {"Name",
            "Full path",
            "Comment",
            "Modification date",
            "Size"};
	private IDirectory myDirectory;
	private IDirectory myArchive;
	private boolean[] myColumnsVisible = {true,
			true, true, true, true};	

	public MainTableModel(IDirectory folder) {
		myArchive = folder;
		myDirectory = folder;	
		myDirectory.addUp();
	}
	
	public void setColumnVisibility(int column, boolean isVisible) {
		myColumnsVisible[column] = isVisible;
		fireTableStructureChanged();
	}
	
	private int getRightColumnNumber (int col) {
        int n = col;    
        int i = 0;
        do {
            if (!(myColumnsVisible[i])) {
            	n++;
            }
            i++;
        } while (i < n);
        while (!(myColumnsVisible[n])) {
        	n++;
        }
        return n;
    }

	public IDirectory getDirectory() {
		return myDirectory;
	}
	
	public IDirectory getArchive() {
		return myArchive;
	}
	
	public void setDirectory(IDirectory directory) {
		myDirectory = directory;
		fireTableChanged(new TableModelEvent(this));
	}
	
	public void setArchive(IDirectory directory) {
		myArchive = directory;		
	}
	
	public String getColumnName(int col) {
        return myColumnNames[getRightColumnNumber(col)].toString();
    }


	public int getRowCount() {	
		return (myDirectory.getFiles().size() + myDirectory.getDirectories().size());
	}
	
	public int getColumnCount() {
	    int n = 0;
	    for (int i = 0; i < 5; i++) {
	        if (myColumnsVisible[i]) {
	            n++;
	        }
	    }
	    return n;
	}
	
	public Class<?> getColumnClass(int column) {
		if(column == 0) {
			return NameView.class;
		}
		if(column == 4) {
			return SizeView.class;
		}
		if(column == 3) {
			return TimeView.class;
		}
		return String.class;		
	}
	
    public Object getValueAt(int row, int col) {
    	final IFile current;  
    	int dirSize = myDirectory.getDirectories().size();
    	int fileSize = myDirectory.getFiles().size();
    	if ((row >= 0) && (row <= dirSize - 1)) {
	    	current = myDirectory.getDirectories().get(row);
    	} else {
	    	if ((row >= dirSize) && (row < dirSize + fileSize)) {
	    		current = myDirectory.getFiles().get(row - dirSize);
	    	} else {
	    		return "";
	    	}
	    }  
		switch(getRightColumnNumber(col)) {
		    case 0: 
		    	NameView curName = new NameView(current);
		    	return curName;	  	   
	    	case 1: 
	    		return current.getFullPath();
	    	case 2: 
	    		return current.getComment();
	    	case 3: 
	    		SimpleDateFormat form = new SimpleDateFormat("dd.MM.yyyy hh:mm:ss");    	
	    		TimeView curTime = new TimeView(current, form);
	    		return curTime; 		 
	    	case 4: 
	    		SizeView curSize = new SizeView(current, SizeConverter.SIZECONVERTER);
	    		return curSize;	    		
	    }
    	return "";
    }   
}