package ru.amse.timkina.archiver.ui.tree;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import ru.amse.timkina.archiver.filetree.IDirectory;
import ru.amse.timkina.archiver.filetree.IFile;

public class FileTreeModel implements TreeModel {
	private IDirectory myDirectory;
	private final List<TreeModelListener> myListeners = new ArrayList<TreeModelListener>();
	
	public FileTreeModel(IDirectory directory) {
		myDirectory = directory;	
	} 
	
	public void setDirectory(IDirectory directory) {
		myDirectory = directory;
		for (TreeModelListener l : myListeners) {
			l.treeStructureChanged(new TreeModelEvent(this, new TreePath(getRoot())));
		}	
	}
	
	public Object getChild(Object parent, int index) {
		IFile parentAsDirectory = (IFile) parent;
		if (!parentAsDirectory.isDirectory()) {
			return null;
		} else {			
			int size = parentAsDirectory.getDirectories().size();
			if (((IDirectory) parentAsDirectory).hasUp()) {
				size --;
			}
			int fileSize = parentAsDirectory.getFiles().size();
			if ((index >= 0) && (index < size)) {
				if (((IDirectory) parentAsDirectory).hasUp()) {
				    index++;
			    }
		    	return parentAsDirectory.getDirectories().get(index);
	    	} else {
		    	if ((index >= size) && (index < size + fileSize)) {
		    		return parentAsDirectory.getFiles().get(index - size);
		    	} else {
		    		return null;
		    	}
		    }    	
		}
	}
	
	public Object getRoot() {
		return myDirectory;
	}
	
	public void removeTreeModelListener(TreeModelListener l) {
		myListeners.remove(l);
	}
	
	public void valueForPathChanged(TreePath path, Object newValue) {
		
	}
	
	public void addTreeModelListener(TreeModelListener l) {
		myListeners.add(l);
	}
	 
	public boolean isLeaf(Object node) {
		return !((IFile) node).isDirectory(); 
	}
	
	public TreePath getPath(IFile node) {
		List<IFile> list = new ArrayList<IFile>(); 
		while (node != getRoot()) {
	        list.add(node);
	        node = node.getParent();
	    }
	    list.add((IDirectory) getRoot());
	    Collections.reverse(list);
	    return new TreePath(list.toArray());
	}
	
	public int getIndexOfChild(Object parent, Object child) {
		IFile parentAsDirectory = (IFile) parent;
		if ((parent == null) || (child == null) || 
				(!parentAsDirectory.isDirectory())) {
			return -1;
		}
		if (((IFile) child).isDirectory()) {
			if (((IDirectory) parentAsDirectory).hasUp()) {
				return parentAsDirectory.getDirectories().indexOf(child) - 1;
			}
			return parentAsDirectory.getDirectories().indexOf(child);
		} else {
			return parentAsDirectory.getFiles().indexOf(child);
		}
	}
	
	public int getChildCount(Object parent) {
		if (isLeaf(parent)) {
			return 0;
		} else {
			IDirectory parentAsDirectory = (IDirectory) parent;
			return parentAsDirectory.getContentSize(); 
		}
	}
}
