package ru.amse.timkina.archiver.zipprocessor;

import java.io.IOException;
import java.util.*;
import java.util.zip.*;
import java.util.Enumeration;
import java.util.GregorianCalendar;
import java.util.ArrayList;
import java.util.HashMap;

import ru.amse.timkina.archiver.filetree.Directory;
import ru.amse.timkina.archiver.filetree.File;
import ru.amse.timkina.archiver.filetree.IDirectory;
import ru.amse.timkina.archiver.filetree.IFile;
public class ZipLoader {
	
	private ZipFile myFile;	
	
	protected  void putDirectories(List<IDirectory> directories, Map<String, 
			IDirectory> dirMap, IDirectory resultDirectory) {
		for (IDirectory i : directories) {
			String[] parts = i.getName().split("/");
			IDirectory result = resultDirectory;
			for (int j = 0; j < parts.length; j++) {
				String shortName = parts[j];
				String longName;
				if (result == resultDirectory) {
					longName = shortName + "/";
				} else {
					longName = result.getName() + shortName + "/";
				}		
				if (!result.containsDirectory(shortName)) {
					IDirectory element = dirMap.get(longName);
					element.setName(longName);
					element.setFullPath(result.getFullPath() + shortName + java.io.File.separator);
					result.add(element);					
					result = element;
				} else {
					result = result.getDirectoryByName(shortName);					
				}
			}
		}
	}
	
	protected  void putFiles(List<IFile> files, Map<String, IDirectory> dirMap, IDirectory resultDirectory) {
		for (IFile i : files) {
			IDirectory result;
			if (i.getName().lastIndexOf('/') != -1) {
				if (i.getName().startsWith("META-INF/")) {
					continue;
				}				
				result = dirMap.get(i.getName().substring(0, i.getName().lastIndexOf('/') + 1));
			} else {
				result = resultDirectory;
			}
			i.setFullPath(result.getFullPath() + i.getShortName());
			result.add(i);
		}
	}
	
	public IDirectory loadFile(String zipFilePath) throws IOException {
		myFile = new ZipFile(zipFilePath);	
		Enumeration<? extends ZipEntry> content = myFile.entries();
		IDirectory resultDirectory = new Directory();
		List<IFile> files = new ArrayList<IFile>();
		List<IDirectory> directories = new ArrayList<IDirectory>();
		resultDirectory.setName(myFile.getName());
		resultDirectory.setFullPath(zipFilePath + java.io.File.separator);
		List<ZipEntry> zipEntries = new ArrayList<ZipEntry>();
		while (content.hasMoreElements()) {
		   zipEntries.add(content.nextElement());
		}		
		for (int i = 0; i < zipEntries.size(); i++) {
		    ZipEntry entry = zipEntries.get(i);	
		    IFile element;
		    if (!entry.isDirectory()) {
		    	IFile el = new File(entry.getSize());
		    	element = el;		    	
				files.add(element);				
		    } else {
		    	IDirectory el = new Directory();
		    	element = el;
			    directories.add((IDirectory)element);
	        }
		    element.setComment(entry.getComment());		  
		    element.setName(entry.getName());		
			GregorianCalendar time = new GregorianCalendar();
			time.setTimeInMillis(entry.getTime());
			element.setModificationTime(time);
			element.setFullPath(zipFilePath);   			
		}
		
	    Map<String, IDirectory> dirMap = new HashMap<String, IDirectory>();		
	    for (int i = 0; i < directories.size(); i++) {
			dirMap.put(directories.get(i).getName(), directories.get(i));
		}
	    putDirectories(directories, dirMap, resultDirectory);
	    putFiles(files, dirMap, resultDirectory);	
		return resultDirectory;
	}
}
