package ru.amse.timkina.archiver.zipprocessor;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.io.FileOutputStream;
import java.io.File;
import java.util.zip.*;
import java.util.Map;

import ru.amse.timkina.archiver.filetree.IDirectory;

public class ZipMaker {
	private List<File> myFiles = new ArrayList<File>();
	private List<File> myDirectories = new ArrayList<File>();
	
	/*package*/ void getFromOne(File f, Map<File, String> fileMap, Map<File, String> dirMap) {
		if (f.isDirectory()) {			
			myDirectories.add(f);			
			for (File file : f.listFiles()) {
				String newDirectoryName = dirMap.get(f) + '/' + file.getName();
				if (file.isDirectory()) {					
					dirMap.put(file, newDirectoryName);
				} else {
					fileMap.put(file, newDirectoryName);					
				}		
				getFromOne(file, fileMap, dirMap);
			}			
		} else {
			myFiles.add(f);
		}
	}
		
	public IDirectory makeZip(File zipFile, File[] files) throws IOException {
		Map<File, String> fileMap = new HashMap<File, String>();
		Map<File, String> dirMap = new HashMap<File, String>();
		fillMaps(files, fileMap, dirMap);
		String path = appendFileExtensionIfNeeded(zipFile);		
		ZipOutputStream writer = new ZipOutputStream(new FileOutputStream(path));			
		try {
		    putDirectories(dirMap, writer);
		    putFiles(fileMap, writer);
		} finally {
			writer.close();
		}		
		IDirectory directory = new ZipLoader().loadFile(path);
		directory.addUp();
		return directory;
	}

	private void putFiles(Map<File, String> fileMap, ZipOutputStream writer)
			throws FileNotFoundException, IOException {
		for (File f : myFiles) {
		    String name = fileMap.get(f);		
		    ZipEntry entry = new ZipEntry(name);			
		    FileInputStream reader = new FileInputStream(f);
		    try {
		        writer.putNextEntry(entry);
		        byte[] buf = new byte[1024];
		        int number;
		        while ((number = reader.read(buf)) >= 0) {
		    	    writer.write(buf, 0 , number);
		        }
		        entry.setTime(f.lastModified());
		    } finally {
			    reader.close();
		    } 
		    writer.closeEntry();		
		}
	}

	private void putDirectories(Map<File, String> dirMap, ZipOutputStream writer)
			throws IOException {
		for (File f : myDirectories) {
		    String name = dirMap.get(f);
		    ZipEntry entry = new ZipEntry(name + '/');
		    writer.putNextEntry(entry);
		}
	}

	private String appendFileExtensionIfNeeded(File zipFile) {
		String path = zipFile.getAbsolutePath();
		if (!zipFile.getName().endsWith(".zip")) {
			path = path + ".zip";
		}
		return path;
	}

	private void fillMaps(File[] files, Map<File, String> fileMap,
			Map<File, String> dirMap) {
		for (int i = 0; i < files.length; i++) {
			if (files[i].isFile()) {
				fileMap.put(files[i], files[i].getName());
			} else {
				dirMap.put(files[i], files[i].getName());
			}
		}		
		for (int i = 0; i < files.length; i++) {			
			getFromOne(files[i], fileMap, dirMap);
		}
	}	
}